@@ -12,10 +12,12 @@
  * other special, indirect and consequential damages.
 */
 #include "clock.h"
-#include <klocale.h>
 #include "deck.h"
 #include "patsolve/clock.h"
-#include <assert.h>
+
+#include <cassert>
+
+#include <klocale.h>
 
 Clock::Clock( )
     : DealerScene( )
@@ -101,7 +103,7 @@ void Clock::deal() {
 static class LocalDealerInfo11 : public DealerInfo
 {
 public:
-    LocalDealerInfo11() : DealerInfo(I18N_NOOP("G&randfather's Clock"), 11) {}
+    LocalDealerInfo11() : DealerInfo(I18N_NOOP("Grandfather's Clock"), 11) {}
     virtual DealerScene *createGame() { return new Clock(); }
 } gfi11;
 
