@@ -11,12 +11,16 @@
  * event will the author be liable for any lost revenue or profits or
  * other special, indirect and consequential damages.
 */
+
 #include "fortyeight.h"
+#include "deck.h"
+#include "speeds.h"
+#include "patsolve/fortyeight.h"
+
+#include <cassert>
+
 #include <klocale.h>
 #include <kdebug.h>
-#include "deck.h"
-#include <assert.h>
-#include <patsolve/fortyeight.h>
 
 HorLeftPile::HorLeftPile( int _index, DealerScene* parent)
     : Pile(_index, parent)
@@ -119,7 +123,7 @@ void Fortyeight::deckPressed(Card *c2)
     qreal x = c->realX();
     qreal y = c->realY();
     c->setPos( Deck::deck()->pos() );
-    c->flipTo(x, y);
+    c->flipTo(x, y, DURATION_FLIP );
     takeState();
 }
 
@@ -171,7 +175,7 @@ void Fortyeight::setGameState( const QString &s )
 static class LocalDealerInfo8 : public DealerInfo
 {
 public:
-    LocalDealerInfo8() : DealerInfo(I18N_NOOP("Forty && &Eight"), 8) {}
+    LocalDealerInfo8() : DealerInfo(I18N_NOOP("Forty && Eight"), 8) {}
     virtual DealerScene *createGame() { return new Fortyeight(); }
 } ldi9;
 
