@@ -1,5 +1,24 @@
-/* Yo Emacs, this -*- C++ -*-
- *******************************************************************
+/*******************************************************************
+ *
+ * Copyright 1997 Mario Weilguni <mweilguni@sime.com>
+ *
+ * This file is part of the KDE project "KREVERSI"
+ *
+ * KREVERSI is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * KREVERSI is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with KREVERSI; see the file COPYING.  If not, write to
+ * the Free Software Foundation, 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
+ *
  *******************************************************************
  *
  *
@@ -21,27 +40,6 @@
  *
  * If you are interested on the JAVA applet of Mr. Luthman take a
  * look at http://www.sylog.se/~mats/
- *
- *******************************************************************
- *
- * This file is part of the KDE project "KREVERSI"
- *
- * KREVERSI is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * KREVERSI is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with KREVERSI; see the file COPYING.  If not, write to
- * the Free Software Foundation, 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
- *
- *******************************************************************
  */
 
 // The class Engine produces moves from a Game object through calls to the
@@ -117,49 +115,10 @@
 // or nearly equal value after the search is completed.
 
 
-#include <qapplication.h>
-
 #include "Engine.h"
-
-
-// ================================================================
-//                          Class ULONG64
-
-
-#if !defined(__GNUC__)
-
-
-ULONG64::ULONG64() : QBitArray(64) 
-{
-  fill(0);
-}
-
-
-// Initialize an ULONG64 from a 32 bit value.
-//
-
-ULONG64::ULONG64( unsigned int value ) : QBitArray(64) 
-{
-  fill(0);
-  for(int i = 0; i < 32; i++) {
-    setBit(i, (bool)(value & 1));
-    value >>= 1;
-  }
-}
-
-
-// Shift an ULONG64 left one bit.
-//
-
-void ULONG64::shl() 
-{
-  for(int i = 63; i > 0; i--)
-    setBit(i, testBit(i - 1));
-  setBit(0, 0);
-}
-
-#endif
-
+#include "kreversigame.h"
+#include <QApplication>
+#include <KDebug>
 
 // ================================================================
 //           Classes SquareStackEntry and SquareStack
@@ -194,7 +153,7 @@ SquareStack::SquareStack(int size) {
 }
 
 
-void SquareStack::resize(int size) 
+void SquareStack::resize(int size)
 {
   m_squarestack.resize(size);
 }
@@ -204,7 +163,7 @@ void SquareStack::resize(int size)
 // resize it to 'size'.
 //
 
-void SquareStack::init(int size) 
+void SquareStack::init(int size)
 {
   resize(size);
 
@@ -215,7 +174,7 @@ void SquareStack::init(int size)
 
 
 
-inline SquareStackEntry SquareStack::Pop() 
+inline SquareStackEntry SquareStack::Pop()
 {
   return m_squarestack[--m_top];
 }
@@ -236,7 +195,7 @@ inline void SquareStack::Push(int x, int y)
 //
 
 
-inline void MoveAndValue::setXYV(int x, int y, int value) 
+inline void MoveAndValue::setXYV(int x, int y, int value)
 {
   m_x     = x;
   m_y     = y;
@@ -244,51 +203,94 @@ inline void MoveAndValue::setXYV(int x, int y, int value)
 }
 
 
-MoveAndValue::MoveAndValue() 
+MoveAndValue::MoveAndValue()
 {
   setXYV(0,0,0);
 }
 
 
-MoveAndValue::MoveAndValue(int x, int y, int value) 
+MoveAndValue::MoveAndValue(int x, int y, int value)
 {
   setXYV(x, y, value);
 }
 
+// ================================================================
+//                       class Score
+
+/* This class keeps track of the score for both colors.  Such a score
+ * could be either the number of pieces, the score from the evaluation
+ * function or anything similar.
+ */
+class Score {
+public:
+  Score()
+  {
+      m_score[White] = 0;
+      m_score[Black] = 0;
+  }
+
+  uint score(ChipColor color) const     { return m_score[color]; }
+
+  void set(ChipColor color, uint score) { m_score[color] = score; }
+  void inc(ChipColor color)             { m_score[color]++; }
+  void dec(ChipColor color)             { m_score[color]--; }
+  void add(ChipColor color, uint s)     { m_score[color] += s; }
+  void sub(ChipColor color, uint s)     { m_score[color] -= s; }
+
+private:
+  uint  m_score[2];
+};
 
 // ================================================================
 //                        The Engine itself
 
 
 // Some special values used in the search.
-const int Engine::LARGEINT      = 99999;
-const int Engine::ILLEGAL_VALUE = 8888888;
-const int Engine::BC_WEIGHT     = 3;
+static const int LARGEINT      = 99999;
+static const int ILLEGAL_VALUE = 8888888;
+static const int BC_WEIGHT     = 3;
 
 
-Engine::Engine(int st, int sd) : SuperEngine(st, sd) 
+Engine::Engine(int st, int sd)/* : SuperEngine(st, sd) */
+    : m_strength(st), m_computingMove( false )
 {
+  m_random.setSeed(sd);
+  m_score = new Score;
+  m_bc_score = new Score;
   SetupBcBoard();
   SetupBits();
 }
 
 
-Engine::Engine(int st) : SuperEngine(st) 
+Engine::Engine(int st) //: SuperEngine(st)
+    : m_strength(st), m_computingMove(false)
 {
+  m_random.setSeed(0);
+  m_score = new Score;
+  m_bc_score = new Score;
   SetupBcBoard();
   SetupBits();
 }
 
 
-Engine::Engine() : SuperEngine(1) 
+Engine::Engine()// : SuperEngine(1)
+    : m_strength(1), m_computingMove(false)
 {
+  m_random.setSeed(0);
+  m_score = new Score;
+  m_bc_score = new Score;
   SetupBcBoard();
   SetupBits();
 }
 
+Engine::~Engine()
+{
+    delete m_score;
+    delete m_bc_score;
+}
 
 // keep GUI alive
-void Engine::yield() 
+void Engine::yield()
 {
   qApp->processEvents();
 }
@@ -296,9 +298,16 @@ void Engine::yield()
 
 // Calculate the best move from the current position, and return it.
 
-Move Engine::computeMove(Game *game, bool competitive) 
+KReversiPos Engine::computeMove(const KReversiGame& game, bool competitive)
 {
-  Color color;
+    if( m_computingMove )
+    {
+        kDebug() << "I'm already computing move! Yours KReversi Engine.";
+        return KReversiPos();
+    }
+    m_computingMove = true;
+
+  ChipColor color;
 
   // A competitive game is one where we try our damnedest to make the
   // best move.  The opposite is a casual game where the engine might
@@ -313,18 +322,24 @@ Move Engine::computeMove(Game *game, bool competitive)
   m_exhaustive = false;
 
   // Get the color to calculate the move for.
-  color = game->toMove();
-  if (color == Nobody)
-    return Move(Nobody, -1, -1);
+  color = game.currentPlayer();
+  if (color == NoColor)
+  {
+      m_computingMove = false;
+      return KReversiPos();
+  }
 
   // Figure out the current score
-  m_score.set(White, game->score(White));
-  m_score.set(Black, game->score(Black));
+  m_score->set(White, game.playerScore(White));
+  m_score->set(Black, game.playerScore(Black));
 
   // Treat the first move as a special case (we can basically just
   // pick a move at random).
-  if (m_score.score(White) + m_score.score(Black) == 4)
-    return ComputeFirstMove(game);
+  if (m_score->score(White) + m_score->score(Black) == 4)
+  {
+      m_computingMove = false;
+      return ComputeFirstMove(game);
+  }
 
   // Let there be room for 3000 changes during the recursive search.
   // This is more than will ever be needed.
@@ -332,18 +347,18 @@ Move Engine::computeMove(Game *game, bool competitive)
 
   // Get the search depth.  If we are close to the end of the game,
   // the number of possible moves goes down, so we can search deeper
-  // without using more time.  
+  // without using more time.
   m_depth = m_strength;
-  if (m_score.score(White) + m_score.score(Black) + m_depth + 3 >= 64)
-    m_depth = 64 - m_score.score(White) - m_score.score(Black);
-  else if (m_score.score(White) + m_score.score(Black) + m_depth + 4 >= 64)
+  if (m_score->score(White) + m_score->score(Black) + m_depth + 3 >= 64)
+    m_depth = 64 - m_score->score(White) - m_score->score(Black);
+  else if (m_score->score(White) + m_score->score(Black) + m_depth + 4 >= 64)
     m_depth += 2;
-  else if (m_score.score(White) + m_score.score(Black) + m_depth + 5 >= 64)
+  else if (m_score->score(White) + m_score->score(Black) + m_depth + 5 >= 64)
     m_depth++;
 
   // If we are very close to the end, we can even make the search
   // exhaustive.
-  if (m_score.score(White) + m_score.score(Black) + m_depth >= 64)
+  if (m_score->score(White) + m_score->score(Black) + m_depth >= 64)
     m_exhaustive = true;
 
   // The evaluation is a linear combination of the score (number of
@@ -352,7 +367,7 @@ Move Engine::computeMove(Game *game, bool competitive)
   // values and the later in the game the more we use the number of
   // pieces.
   m_coeff = 100 - (100*
-		   (m_score.score(White) + m_score.score(Black) 
+		   (m_score->score(White) + m_score->score(Black)
 		    + m_depth - 4)) / 60;
 
   // Initialize the board that we use for the search.
@@ -360,9 +375,9 @@ Move Engine::computeMove(Game *game, bool competitive)
     for (uint y = 0; y < 10; y++) {
       if (1 <= x && x <= 8
 	  && 1 <= y && y <= 8)
-	m_board[x][y] = game->color(x, y);
+	m_board[x][y] = game.chipColorAt(y-1, x-1);
       else
-	m_board[x][y] = Nobody;
+	m_board[x][y] = NoColor;
     }
 
   // Initialize a lot of stuff that we will use in the search.
@@ -370,11 +385,11 @@ Move Engine::computeMove(Game *game, bool competitive)
   // Initialize m_bc_score to the current bc score.  This is kept
   // up-to-date incrementally so that way we won't have to calculate
   // it from scratch for each evaluation.
-  m_bc_score.set(White, CalcBcScore(White));
-  m_bc_score.set(Black, CalcBcScore(Black));
+  m_bc_score->set(White, CalcBcScore(White));
+  m_bc_score->set(Black, CalcBcScore(Black));
 
-  ULONG64 colorbits    = ComputeOccupiedBits(color);
-  ULONG64 opponentbits = ComputeOccupiedBits(opponent(color));
+  quint64 colorbits    = ComputeOccupiedBits(color);
+  quint64 opponentbits = ComputeOccupiedBits(opponentColorFor(color));
 
   int maxval = -LARGEINT;
   int max_x = 0;
@@ -386,18 +401,18 @@ Move Engine::computeMove(Game *game, bool competitive)
 
   setInterrupt(false);
 
-  ULONG64 null_bits;
+  quint64 null_bits;
   null_bits = 0;
 
   // The main search loop.  Step through all possible moves and keep
-  // track of the most valuable one.  This move is stored in 
+  // track of the most valuable one.  This move is stored in
   // (max_x, max_y) and the value is stored in maxval.
   m_nodes_searched = 0;
   for (int x = 1; x < 9; x++) {
     for (int y = 1; y < 9; y++) {
       // Don't bother with non-empty squares and squares that aren't
       // neighbors to opponent pieces.
-      if (m_board[x][y] != Nobody
+      if (m_board[x][y] != NoColor
 	  || (m_neighbor_bits[x][y] & opponentbits) == null_bits)
 	continue;
 
@@ -418,8 +433,8 @@ Move Engine::computeMove(Game *game, bool competitive)
 	  // user wants a casual game, which is set in the settings
 	  // dialog.
 	  int randi = m_random.getLong(7);
-	  if (maxval == -LARGEINT 
-	      || m_competitive 
+	  if (maxval == -LARGEINT
+	      || m_competitive
 	      || randi < (int) m_strength) {
 	    maxval = val;
 	    max_x  = x;
@@ -433,7 +448,7 @@ Move Engine::computeMove(Game *game, bool competitive)
       }
 
       // Jump out prematurely if interrupt is set.
-      if (interrupted()) 
+      if (interrupted())
 	break;
     }
   }
@@ -446,7 +461,7 @@ Move Engine::computeMove(Game *game, bool competitive)
     int  i;
 
     for (i = 0; i < number_of_moves; i++) {
-      if (moves[i].m_value == maxval && --r <= 0) 
+      if (moves[i].m_value == maxval && --r <= 0)
 	break;
     }
 
@@ -454,37 +469,38 @@ Move Engine::computeMove(Game *game, bool competitive)
     max_y = moves[i].m_y;
   }
 
+  m_computingMove = false;
   // Return a suitable move.
   if (interrupted())
-    return Move(Nobody, -1, -1);
+    return KReversiPos(NoColor, -1, -1);
   else if (maxval != -LARGEINT)
-    return Move(color, max_x, max_y);
+    return KReversiPos(color, max_y-1, max_x-1);
   else
-    return Move(Nobody, -1, -1);
+    return KReversiPos(NoColor, -1, -1);
 }
 
 
 // Get the first move.  We can pick any move at random.
 //
 
-Move Engine::ComputeFirstMove(Game *game) 
+KReversiPos Engine::ComputeFirstMove(const KReversiGame& game)
 {
   int    r;
-  Color  color = game->toMove();
+  ChipColor  color = game.currentPlayer();
 
   r = m_random.getLong(4) + 1;
 
   if (color == White) {
-    if (r == 1)      return  Move(color, 3, 5);
-    else if (r == 2) return  Move(color, 4, 6);
-    else if (r == 3) return  Move(color, 5, 3);
-    else             return  Move(color, 6, 4);
+    if (r == 1)      return  KReversiPos(color, 4, 2);
+    else if (r == 2) return  KReversiPos(color, 5, 3);
+    else if (r == 3) return  KReversiPos(color, 2, 4);
+    else             return  KReversiPos(color, 3, 5);
   }
   else {
-    if (r == 1)      return  Move(color, 3, 4);
-    else if (r == 2) return  Move(color, 5, 6);
-    else if (r == 3) return  Move(color, 4, 3);
-    else             return  Move(color, 6, 5);
+    if (r == 1)      return  KReversiPos(color, 3, 2);
+    else if (r == 2) return  KReversiPos(color, 5, 4);
+    else if (r == 3) return  KReversiPos(color, 2, 3);
+    else             return  KReversiPos(color, 4, 5);
   }
 }
 
@@ -495,26 +511,26 @@ Move Engine::ComputeFirstMove(Game *game)
 // search.
 //
 
-int Engine::ComputeMove2(int xplay, int yplay, Color color, int level,
-			 int cutoffval, ULONG64 colorbits,
-			 ULONG64 opponentbits)
+int Engine::ComputeMove2(int xplay, int yplay, ChipColor color, int level,
+			 int cutoffval, quint64 colorbits,
+			 quint64 opponentbits)
 {
   int               number_of_turned = 0;
   SquareStackEntry  mse;
-  Color             opponent = ::opponent(color);
+  ChipColor             opponent = opponentColorFor(color);
 
   m_nodes_searched++;
 
   // Put the piece on the board and incrementally update scores and bitmaps.
   m_board[xplay][yplay] = color;
   colorbits |= m_coord_bit[xplay][yplay];
-  m_score.inc(color);
-  m_bc_score.add(color, m_bc_board[xplay][yplay]);
+  m_score->inc(color);
+  m_bc_score->add(color, m_bc_board[xplay][yplay]);
 
   // Loop through all 8 directions and turn the pieces that can be turned.
   for (int xinc = -1; xinc <= 1; xinc++)
     for (int yinc = -1; yinc <= 1; yinc++) {
-      if (xinc == 0 && yinc == 0) 
+      if (xinc == 0 && yinc == 0)
 	continue;
 
       int x, y;
@@ -536,8 +552,8 @@ int Engine::ComputeMove2(int xplay, int yplay, Color color, int level,
 
 	  m_squarestack.Push(x, y);
 
-	  m_bc_score.add(color, m_bc_board[x][y]);
-	  m_bc_score.sub(opponent, m_bc_board[x][y]);
+	  m_bc_score->add(color, m_bc_board[x][y]);
+	  m_bc_score->sub(opponent, m_bc_board[x][y]);
 	  number_of_turned++;
 	}
     }
@@ -549,8 +565,8 @@ int Engine::ComputeMove2(int xplay, int yplay, Color color, int level,
   if (number_of_turned > 0) {
 
     // First adjust the number of pieces for each side.
-    m_score.add(color, number_of_turned);
-    m_score.sub(opponent, number_of_turned);
+    m_score->add(color, number_of_turned);
+    m_score->sub(opponent, number_of_turned);
 
     // If we are at the bottom of the search, get the evaluation.
     if (level >= m_depth)
@@ -569,14 +585,14 @@ int Engine::ComputeMove2(int xplay, int yplay, Color color, int level,
 	if (retval == -LARGEINT) {
 
 	  // No possible move for anybody => end of game:
-	  int finalscore = m_score.score(color) - m_score.score(opponent);
+	  int finalscore = m_score->score(color) - m_score->score(opponent);
 
 	  if (m_exhaustive)
 	    retval = finalscore;
 	  else {
 	    // Take a sure win and avoid a sure loss (may not be optimal):
 
-	    if (finalscore > 0) 
+	    if (finalscore > 0)
 	      retval = LARGEINT - 65 + finalscore;
 	    else if (finalscore < 0)
 	      retval = -(LARGEINT - 65 + finalscore);
@@ -587,22 +603,22 @@ int Engine::ComputeMove2(int xplay, int yplay, Color color, int level,
       }
     }
 
-    m_score.add(opponent, number_of_turned);
-    m_score.sub(color, number_of_turned);
+    m_score->add(opponent, number_of_turned);
+    m_score->sub(color, number_of_turned);
   }
 
   // Undo the move.  Start by unturning the turned pieces.
   for (int i = number_of_turned; i > 0; i--) {
     mse = m_squarestack.Pop();
-    m_bc_score.add(opponent, m_bc_board[mse.m_x][mse.m_y]);
-    m_bc_score.sub(color, m_bc_board[mse.m_x][mse.m_y]);
+    m_bc_score->add(opponent, m_bc_board[mse.m_x][mse.m_y]);
+    m_bc_score->sub(color, m_bc_board[mse.m_x][mse.m_y]);
     m_board[mse.m_x][mse.m_y] = opponent;
   }
 
   // Now remove the new piece that we put down.
-  m_board[xplay][yplay] = Nobody;
-  m_score.sub(color, 1);
-  m_bc_score.sub(color, m_bc_board[xplay][yplay]);
+  m_board[xplay][yplay] = NoColor;
+  m_score->sub(color, 1);
+  m_bc_score->sub(color, m_bc_board[xplay][yplay]);
 
   // Return a suitable value.
   if (number_of_turned < 1 || interrupted())
@@ -617,27 +633,27 @@ int Engine::ComputeMove2(int xplay, int yplay, Color color, int level,
 // most valuable move, but not the move itself.
 //
 
-int Engine::TryAllMoves(Color opponent, int level, int cutoffval,
-			ULONG64 opponentbits, ULONG64 colorbits)
+int Engine::TryAllMoves(ChipColor opponent, int level, int cutoffval,
+			quint64 opponentbits, quint64 colorbits)
 {
   int maxval = -LARGEINT;
 
   // Keep GUI alive by calling the event loop.
   yield();
 
-  ULONG64  null_bits;
+  quint64  null_bits;
   null_bits = 0;
 
   for (int x = 1; x < 9; x++) {
     for (int y = 1; y < 9; y++) {
-      if (m_board[x][y] == Nobody 
+      if (m_board[x][y] == NoColor
 	  && (m_neighbor_bits[x][y] & colorbits) != null_bits) {
 	int val = ComputeMove2(x, y, opponent, level+1, maxval, opponentbits,
 			       colorbits);
 
 	if (val != ILLEGAL_VALUE && val > maxval) {
 	  maxval = val;
-	  if (maxval > -cutoffval || interrupted()) 
+	  if (maxval > -cutoffval || interrupted())
 	    break;
 	}
       }
@@ -647,7 +663,7 @@ int Engine::TryAllMoves(Color opponent, int level, int cutoffval,
       break;
   }
 
-  if (interrupted()) 
+  if (interrupted())
     return -LARGEINT;
 
   return maxval;
@@ -660,22 +676,22 @@ int Engine::TryAllMoves(Color opponent, int level, int cutoffval,
 // using the board control values.
 //
 
-int Engine::EvaluatePosition(Color color)
+int Engine::EvaluatePosition(ChipColor color)
 {
   int retval;
 
-  Color  opponent = ::opponent(color);
+  ChipColor opponent = opponentColorFor(color);
 
-  int    score_color    = m_score.score(color);
-  int    score_opponent = m_score.score(opponent);
+  int    score_color    = m_score->score(color);
+  int    score_opponent = m_score->score(opponent);
 
   if (m_exhaustive)
     retval = score_color - score_opponent;
   else {
     retval = (100-m_coeff) *
-      (m_score.score(color) - m_score.score(opponent)) 
-      + m_coeff * BC_WEIGHT * (m_bc_score.score(color)
-			       - m_bc_score.score(opponent));
+      (m_score->score(color) - m_score->score(opponent))
+      + m_coeff * BC_WEIGHT * (m_bc_score->score(color)
+			       - m_bc_score->score(opponent));
   }
 
   return retval;
@@ -691,18 +707,14 @@ void Engine::SetupBits()
   //m_coord_bit = new long[9][9];
   //m_neighbor_bits = new long[9][9];
 
-  ULONG64 bits = 1;
+  quint64 bits = 1;
 
   // Store a 64 bit unsigned it with the corresponding bit set for
   // each square.
   for (int i=1; i < 9; i++)
     for (int j=1; j < 9; j++) {
       m_coord_bit[i][j] = bits;
-#if !defined(__GNUC__)
-      bits.shl();
-#else
       bits *= 2;
-#endif
     }
 
   // Store a bitmap consisting of all neighbors for each square.
@@ -758,7 +770,7 @@ void Engine::SetupBcBoard()
 // Calculate the board control score.
 //
 
-int Engine::CalcBcScore(Color color)
+int Engine::CalcBcScore(ChipColor color)
 {
   int sum = 0;
 
@@ -774,9 +786,9 @@ int Engine::CalcBcScore(Color color)
 // Calculate a bitmap of the occupied squares for a certain color.
 //
 
-ULONG64 Engine::ComputeOccupiedBits(Color color)
+quint64 Engine::ComputeOccupiedBits(ChipColor color)
 {
-  ULONG64 retval = 0;
+  quint64 retval = 0;
 
   for (int i=1; i < 9; i++)
     for (int j=1; j < 9; j++)
