@@ -1,6 +1,6 @@
 /***************************************************************************
  *   Copyright (C) 1999-2006 by Éric Bischoff <ebischoff@nerim.net>        *
- *   Copyright (C) 2007 by Albert Astals Cid <aacid@kde.org>               *
+ *   Copyright (C) 2007-2008 by Albert Astals Cid <aacid@kde.org>          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -72,7 +72,7 @@ void PlayGround::reset()
 // Save objects laid down on the editable area
 bool PlayGround::saveAs(const QString & name)
 {
-  QFile f(QFile::encodeName(name));
+  QFile f(name);
   if (!f.open( QIODevice::WriteOnly | QIODevice::Text ) )
       return false;
 
@@ -104,9 +104,9 @@ bool PlayGround::printPicture(QPrinter &printer)
 // Get a pixmap containing the current picture
 QPixmap PlayGround::getPicture()
 {
-  QPixmap result(mapFromScene(sceneRect()).boundingRect().size());
+  QPixmap result(mapFromScene(backgroundRect()).boundingRect().size());
   QPainter artist(&result);
-  scene()->render(&artist);
+  scene()->render(&artist, QRectF(), backgroundRect());
   artist.end();
   return result;
 }
@@ -160,7 +160,7 @@ void PlayGround::mousePressEvent(QMouseEvent *event)
     }
     else
     {
-      // see if the user clicked on an already existant item
+      // see if the user clicked on an already existent item
       QGraphicsItem *dragItem = m_scene->itemAt(mapToScene(event->pos()));
       m_dragItem = qgraphicsitem_cast<ToDraw*>(dragItem);
       if (m_dragItem)
@@ -384,7 +384,7 @@ QString PlayGround::currentGameboard() const
 // Load objects and lay them down on the editable area
 PlayGround::LoadError PlayGround::loadFrom(const QString &name)
 {
-  QFile f( QFile::encodeName(name) );
+  QFile f(name);
   if (!f.open(QIODevice::ReadOnly | QIODevice::Text))
       return OtherError;
 
