@@ -34,6 +34,7 @@ this software.
 #include <KConfigGroup>
 #include <KTabWidget>
 #include <KDebug>
+#include <KLineEdit>
 
 #include <QtCore/QTimer>
 #include <QtCore/QList>
@@ -42,7 +43,6 @@ this software.
 #include <QtGui/QKeyEvent>
 #include <QtGui/QLabel>
 #include <QtGui/QLayout>
-#include <QtGui/QLineEdit>
 #include <QtGui/QStackedWidget>
 
 #define DEFAULT_GROUP_NAME I18N_NOOP("High Scores")
@@ -54,10 +54,11 @@ class KScoreDialog::KScoreDialogPrivate
     public:
         //QList<FieldInfo*> scores;
         QMap<QByteArray, GroupScores> scores; ///<Maps config group name to GroupScores
+        QMap<int, QByteArray> configGroupWeights; /// Weights of the groups, defines ordering
         KTabWidget *tabWidget;
         //QWidget *page;
         //QGridLayout *layout;
-        QLineEdit *edit;    ///<The line edit for entering player name
+        KLineEdit *edit;    ///<The line edit for entering player name
         QMap<QByteArray, QList<QStackedWidget*> > stack;
         QMap<QByteArray, QList<QLabel*> > labels; ///<For each group, the labels along each row in turn starting from "#1"
         QLabel *commentLabel;
@@ -67,29 +68,29 @@ class KScoreDialog::KScoreDialogPrivate
         QPair<QByteArray, int> newName; //index of the newname to add (groupKey, position)
         QPair<QByteArray, int> latest; //index of the latest addition (groupKey, position)
         int nrCols;
-        int numberOfPages;
         bool loaded;
         QByteArray configGroup;
         KHighscore* highscoreObject;
         QMap<QByteArray, QString> translatedGroupNames; ///<List of the translated group names.
-        
+        QMap<QByteArray, QWidget*> tabs;
+
         QMap<int, int> col;
         QMap<int, QString> header; ///<Header for fields. Maps field index to a string
         QMap<int, QString> key; ///<Keys for fields. Maps field index to a string
         QString player;
-        
+
         //Q-Pointer
         KScoreDialogPrivate(KScoreDialog* parent):q(parent){}
         KScoreDialog* const q;
-        
+
         //Functions
         void loadScores();
         void saveScores();
-        
+
         void setupDialog();
         void setupGroup(const QByteArray& groupName);
         void aboutToShow();
-        
+
         QString findTranslatedGroupName(const QByteArray& name);
 };
 
@@ -108,9 +109,8 @@ KScoreDialog::KScoreDialog(int fields, QWidget *parent)
     d->latest = QPair<QByteArray,int>("Null",-1);
     d->loaded = false;
     d->nrCols = 0;
-    d->numberOfPages=0;
     d->configGroup=QByteArray();
-    
+
     //Set up the default table headers
     d->header[Name] = i18n("Name");
     d->key[Name] = "Name";
@@ -122,12 +122,12 @@ KScoreDialog::KScoreDialog(int fields, QWidget *parent)
     d->key[Score] = "Score";
     d->header[Time] = i18n("Time");
     d->key[Time] = "Time";
-    
+
     //d->page = new QWidget(this);
-    
+
     d->tabWidget = new KTabWidget(this);
     d->tabWidget->setTabPosition(QTabWidget::West);
-    
+
     setMainWidget(d->tabWidget);
     if(d->newName.second == -1)
       setButtons(Close);
@@ -184,6 +184,11 @@ void KScoreDialog::addLocalizedConfigGroupNames(const QMap<QByteArray, QString>&
     }
 }
 
+void KScoreDialog::setConfigGroupWeights(const QMap<int, QByteArray>& weights)
+{
+    d->configGroupWeights = weights;
+}
+
 QString KScoreDialog::KScoreDialogPrivate::findTranslatedGroupName(const QByteArray& name)
 {
     foreach(const QByteArray& groupKey, translatedGroupNames.keys())
@@ -226,80 +231,92 @@ void KScoreDialog::KScoreDialogPrivate::setupDialog()
         if ( (fields & field) && !(hiddenFields & field ) )
             col[field] = nrCols++;
     }
-    
+
     tabWidget->clear();
-    foreach(const QByteArray &groupName, scores.keys())
+    QList<QByteArray> keysToConfigure = scores.keys();
+    foreach(const QByteArray &groupName, configGroupWeights)
+    {
+        int index = keysToConfigure.indexOf(groupName);
+        if (index != -1)
+        {
+            setupGroup(groupName);
+            keysToConfigure.removeAt(index);
+        }
+    }
+    foreach(const QByteArray &groupName, keysToConfigure)
+    {
         setupGroup(groupName);
+    }
 }
 
 void KScoreDialog::KScoreDialogPrivate::setupGroup(const QByteArray& groupKey)
 {
-    if(groupKey.isEmpty()) //If the group doesn't have a name, use a default.
-            tabWidget->addTab(new QWidget(q), i18n(DEFAULT_GROUP_NAME));
-        else
-            tabWidget->addTab(new QWidget(q), findTranslatedGroupName(groupKey));
-        tabWidget->setCurrentIndex(tabWidget->count()-1);
-        
-        QGridLayout* layout = new QGridLayout( tabWidget->widget( tabWidget->currentIndex() ) );
-        //layout->setObjectName("ScoreTab-"+groupName);
-        layout->setMargin(marginHint()+20);
-        layout->setSpacing(spacingHint());
-        layout->addItem(new QSpacerItem(0, 15), 4, 0);
-        
-        commentLabel = new QLabel(tabWidget);
-        commentLabel->setAlignment(Qt::AlignVCenter | Qt::AlignHCenter);
-        
-        QFont bold = q->font();
-        bold.setBold(true);
-        
+    QWidget* widget = new QWidget(q);
+    tabs[groupKey] = widget;
+
+    QString tabName = groupKey.isEmpty() ? i18n(DEFAULT_GROUP_NAME) : findTranslatedGroupName(groupKey);
+    tabWidget->addTab(widget, tabName);
+
+    QGridLayout* layout = new QGridLayout(widget);
+    //layout->setObjectName("ScoreTab-"+groupName);
+    layout->setMargin(marginHint()+20);
+    layout->setSpacing(spacingHint());
+    layout->addItem(new QSpacerItem(0, 15), 4, 0);
+
+    commentLabel = new QLabel(tabWidget);
+    commentLabel->setAlignment(Qt::AlignVCenter | Qt::AlignHCenter);
+
+    QFont bold = q->font();
+    bold.setBold(true);
+
+    QLabel *label;
+    layout->addItem(new QSpacerItem(50, 0), 0, 0);
+    label = new QLabel(i18n("Rank"), widget);
+    layout->addWidget(label, 3, 0);
+    label->setFont(bold);
+
+    for(int field = 1; field < fields; field = field * 2)
+    {
+        if ( (fields & field) && !(hiddenFields & field ) ) //If it's used and not hidden
+        {
+            layout->addItem( new QSpacerItem( 50, 0 ), 0, col[field] );
+            label = new QLabel(header[field], widget);
+            layout->addWidget(label, 3, col[field], field <= Name ? Qt::AlignLeft : Qt::AlignRight);
+            label->setFont(bold);
+        }
+    }
+
+    KSeparator *sep = new KSeparator(Qt::Horizontal, tabWidget->widget(tabWidget->currentIndex()));
+    layout->addWidget(sep, 4, 0, 1, nrCols);
+
+    QString num;
+    for (int i = 1; i <= 10; ++i)
+    {
         QLabel *label;
-        layout->addItem(new QSpacerItem(50, 0), 0, 0);
-        label = new QLabel(i18n("Rank"), tabWidget->widget(tabWidget->currentIndex()));
-        layout->addWidget(label, 3, 0);
-        label->setFont(bold);
-        
-        for(int field = 1; field < fields; field = field * 2)
+        num.setNum(i);
+        label = new QLabel(i18n("#%1", num), widget);
+        labels[groupKey].insert((i-1)*nrCols + 0, label); //Fill up column zero
+        layout->addWidget(label, i+4, 0);
+        if (fields & Name) //If we have a Name field
         {
-            if ( (fields & field) && !(hiddenFields & field ) ) //If it's used and not hidden
+            QStackedWidget *localStack = new QStackedWidget(widget);
+            stack[groupKey].insert(i-1, localStack);
+            layout->addWidget(localStack, i+4, col[Name]);
+            label = new QLabel(localStack);
+            labels[groupKey].insert((i-1)*nrCols + col[Name], label);
+            localStack->addWidget(label);
+            localStack->setCurrentWidget(label);
+        }
+        for(int field = Name * 2; field < fields; field = field * 2)
+        {
+            if ( (fields & field) && !(hiddenFields & field ) ) //Maybe disable for Name?
             {
-                layout->addItem( new QSpacerItem( 50, 0 ), 0, col[field] );
-                label = new QLabel(header[field], tabWidget->widget(tabWidget->currentIndex()));
-                layout->addWidget(label, 3, col[field], field <= Name ? Qt::AlignLeft : Qt::AlignRight);
-                label->setFont(bold);
-            }
-        }
-        
-        KSeparator *sep = new KSeparator(Qt::Horizontal, tabWidget->widget(tabWidget->currentIndex()));
-        layout->addWidget(sep, 4, 0, 1, nrCols);
-        
-        QString num;
-        for (int i = 1; i <= 10; ++i) 
-        {
-            QLabel *label;
-            num.setNum(i);
-            label = new QLabel(i18n("#%1", num), tabWidget->widget(tabWidget->currentIndex()));
-            labels[groupKey].insert((i-1)*nrCols + 0, label); //Fill up column zero
-            layout->addWidget(label, i+4, 0);
-            if (fields & Name) //If we have a Name field
-            {
-                QStackedWidget *localStack = new QStackedWidget(tabWidget->widget(tabWidget->currentIndex()));
-                stack[groupKey].insert(i-1, localStack);
-                layout->addWidget(localStack, i+4, col[Name]);
-                label = new QLabel(localStack);
-                labels[groupKey].insert((i-1)*nrCols + col[Name], label);
-                localStack->addWidget(label);
-                localStack->setCurrentWidget(label);
-            }
-            for(int field = Name * 2; field < fields; field = field * 2)
-            {
-                if ( (fields & field) && !(hiddenFields & field ) ) //Maybe disable for Name?
-                {
-                    label = new QLabel(tabWidget->widget(tabWidget->currentIndex()));
-                    labels[groupKey].insert((i-1)*nrCols + col[field], label);
-                    layout->addWidget(label, i+4, col[field], Qt::AlignRight);
-                }
+                label = new QLabel(widget);
+                labels[groupKey].insert((i-1)*nrCols + col[field], label);
+                layout->addWidget(label, i+4, col[field], Qt::AlignRight);
             }
         }
+    }
 }
 
 /*
@@ -310,30 +327,31 @@ void KScoreDialog::KScoreDialogPrivate::aboutToShow()
 {
     if (!loaded)
         loadScores();
-    
+
     if (!nrCols)
         setupDialog();
-    
+
     int tabIndex=0; //Index of the current tab
-    int newScoreTabIndex=0; //The index of the tab of the group with the new score
     foreach(const QByteArray &groupKey, scores.keys())
     {
+        kDebug() << latest.first << tabWidget->tabText(tabIndex);
+
         //Only display the comment on the page with the new score (or) this one if there's only one tab
-        if((latest.first == tabWidget->tabText(tabIndex)) || ( latest.first.isEmpty() && tabWidget->tabText(tabIndex) == i18n(DEFAULT_GROUP_NAME) ))
+        if(latest.first == groupKey || ( latest.first.isEmpty() && groupKey == DEFAULT_GROUP_NAME ) )
         {
-            newScoreTabIndex=tabIndex;
+            QWidget* widget = tabs.value(groupKey);
+            QGridLayout* layout = qobject_cast<QGridLayout*>(widget->layout());
+
             commentLabel->setText(comment);
             if (comment.isEmpty())
             {
                 commentLabel->setMinimumSize(QSize(1,1));
                 commentLabel->hide();
-                QGridLayout* layout = qobject_cast<QGridLayout*>(tabWidget->widget(newScoreTabIndex)->layout());
                 layout->addItem( new QSpacerItem( 0, -15 ), 0, 0 );
                 layout->addItem( new QSpacerItem( 0, -15 ), 2, 0 );
             }
             else
             {
-                QGridLayout* layout = qobject_cast<QGridLayout*>(tabWidget->widget(newScoreTabIndex)->layout());
                 layout->addWidget(commentLabel, 1, 0, 1, nrCols);
                 commentLabel->setMinimumSize(commentLabel->sizeHint());
                 commentLabel->show();
@@ -341,33 +359,35 @@ void KScoreDialog::KScoreDialogPrivate::aboutToShow()
                 layout->addItem( new QSpacerItem( 0, 10 ), 2, 0 );
             }
             comment.clear();
+
+            tabWidget->setCurrentWidget(widget);
         }
-        
+
         QFont normal = q->font();
         QFont bold = normal;
         bold.setBold(true);
-        
+
         QString num;
         for (int i = 1; i <= 10; ++i)
         {
             QLabel *label;
             num.setNum(i);
-            
+
             //kDebug() << "groupName:" << groupName << "id:" << i-1;
-            
+
             FieldInfo score = scores[groupKey].at(i-1);
             label = labels[groupKey].at((i-1)*nrCols + 0); //crash! FIXME
             if ( (i == latest.second) && (groupKey == latest.first) )
                 label->setFont(bold);
             else
                 label->setFont(normal);
-    
+
             if (fields & Name)
             {
                 if ( (newName.second == i) && (groupKey == newName.first) )
                 {
                     QStackedWidget *localStack = stack[groupKey].at(i-1);
-                    edit = new QLineEdit(player, localStack);
+                    edit = new KLineEdit(player, localStack);
                     edit->setMinimumWidth(40);
                     localStack->addWidget(edit);
                     localStack->setCurrentWidget(edit);
@@ -383,7 +403,7 @@ void KScoreDialog::KScoreDialogPrivate::aboutToShow()
                         label->setFont(normal);
                     label->setText(score[Name]);
                 }
-        
+
             }
             for(int field = Name * 2; field < fields; field = field * 2)
             {
@@ -402,36 +422,32 @@ void KScoreDialog::KScoreDialogPrivate::aboutToShow()
     }
     latest = QPair<QByteArray,int>(QByteArray(),-1);
     q->setFixedSize(q->minimumSizeHint()); //NOTE Remove this line to make dialog resizable
-    tabWidget->setCurrentIndex(newScoreTabIndex);
 }
 
 void KScoreDialog::KScoreDialogPrivate::loadScores()
 {
     scores.clear();
-    
-    QStringList groupList = highscoreObject->groupList(); //List of the group names actually in the config file
-    numberOfPages = groupList.size(); //how many groups are in the config fileo
-    
+
     QList<QByteArray> groupKeyList; //This will be a list of all the groups in the config fie
-    for(int i = 0; i < numberOfPages; i++)
+    foreach( const QString & groupString, highscoreObject->groupList())
     {
-        groupKeyList << groupList.at(i).toUtf8(); //Convert all the QStrings to QByteArrays
+        groupKeyList << groupString.toUtf8(); //Convert all the QStrings to QByteArrays
     }
-    
+
     QByteArray tempCurrentGroup = configGroup; //temp to store the user-set group name
-    
-    if (groupKeyList.count(configGroup) == 0) //If the current group doesn't have any entries, add it to the list to process
+
+    if (!groupKeyList.contains(configGroup)) //If the current group doesn't have any entries, add it to the list to process
     {
         kDebug(11002) << "The current high score group " << configGroup << " isn't in the list, adding it";
         groupKeyList << configGroup;
         setupGroup(configGroup);
     }
-    
+
     foreach(const QByteArray &groupKey, groupKeyList)
     {
         highscoreObject->setHighscoreGroup(groupKey);
         player = highscoreObject->readEntry(0, "LastPlayer");  //FIXME
-        
+
         for (int i = 1; i <= 10; ++i)
         {
             FieldInfo score;
@@ -460,9 +476,9 @@ void KScoreDialog::KScoreDialogPrivate::loadScores()
 void KScoreDialog::KScoreDialogPrivate::saveScores()
 {
     highscoreObject->setHighscoreGroup(configGroup);
-    
+
     highscoreObject->writeEntry(0,"LastPlayer", player);
-    
+
     for (int i = 1; i <= 10; ++i)
     {
         FieldInfo score = scores[configGroup].at(i-1);
@@ -480,45 +496,43 @@ void KScoreDialog::KScoreDialogPrivate::saveScores()
 int KScoreDialog::addScore(const FieldInfo& newInfo, const AddScoreFlags& flags)
 {
     kDebug() << "adding new score";
-    
+
     bool askName=false, lessIsMore=false;
     if(flags.testFlag(KScoreDialog::AskName))
         askName = true;
     if(flags.testFlag(KScoreDialog::LessIsMore))
         lessIsMore = true;
-    
+
     d->latest.first = d->configGroup; //Temporarily set this so loadScores() knows not to delete this group
     if (!d->loaded)
         d->loadScores();
     d->latest.first = "Null"; //and reset it.
-    
+
     for(int i=0; i<d->scores[d->configGroup].size(); i++)
     {
-        kDebug() << "in loop 1";
         FieldInfo score = d->scores[d->configGroup].at(i); //First look at the score in the config file
         bool ok; //will be false if there isn't any score yet in position i
         int num_score = score[Score].toLong(&ok); //test if the stored score is a number
-        
+
         score = FieldInfo(newInfo); //now look at the submitted score
         int newScore = score[Score].toInt();
-        
+
         kDebug() << "num_score =" << num_score << " - newScore =" << newScore;
-        
+
         if (((newScore > num_score) && !lessIsMore) ||
               ((newScore < num_score) && lessIsMore) || !ok)
         {
-            kDebug() << "in if() 1";
             d->latest = QPair<QByteArray,int>(d->configGroup,i+1);
             d->scores[d->configGroup].insert(i, score);
             d->scores[d->configGroup].removeAt(10);
-            
+
             if(score[Name].isEmpty()) //If we don't have a name, prompt the player.
             {
                 if(!d->player.isEmpty()) //d->player should be filled out by d->loadScores()
                 {
                     score[Name] = d->player;
-                } 
-                else 
+                }
+                else
                 {
                     KUser user;
                     score[Name] = user.property(KUser::FullName).toString();
@@ -529,18 +543,18 @@ int KScoreDialog::addScore(const FieldInfo& newInfo, const AddScoreFlags& flags)
                 }
                 askName = true;
             }
-            
+
             if (askName)
             {
                 d->player=score[Name];
                 d->newName = QPair<QByteArray,int>(d->configGroup,i+1);
-                
+
                 setButtons(Ok|Cancel);
                 connect(this, SIGNAL(okClicked()), SLOT(slotGotName()));
             }
             else
                 d->saveScores();
-            
+
             if (i == 0)
                 d->comment = i18n("Excellent!\nYou have a new high score!");
             else
@@ -581,13 +595,13 @@ void KScoreDialog::slotGotName()
     if (d->newName.second == -1) return;
 
     d->player = d->edit->text();
-    
+
     d->scores[d->newName.first][d->newName.second-1][Name] = d->player;
     d->saveScores();
-    
+
     QFont bold = font();
     bold.setBold(true);
-    
+
     QLabel *label = d->labels[d->newName.first].at((d->newName.second-1)*d->nrCols + d->col[Name]);
     label->setFont(bold);
     label->setText(d->player);
@@ -601,7 +615,7 @@ int KScoreDialog::highScore()
 {
     if (!d->loaded)
         d->loadScores();
-   
+
     if (!d->scores[d->configGroup].isEmpty())
         return d->scores[d->configGroup].first()[Score].toInt();
     else
